#include <stdio.h>
#include <et/common.h>
#include "box.h"

// Default size and color of an etBox.
const etVector DEFAULT_SIZE(0.05, 0.05, 0.05); // 50mm.

// Returns the force to be applied.  See etForceRenderer.
etVector etBox::computePointForce(etVector l_pos) {
	if(!cull(l_pos)) {
		etVector l_size = this->getSizeRaw().scale(0.5);
		
		double l_distance[6];
		l_distance[0] = l_size.x() - l_pos.x();
		l_distance[5] = l_size.x() + l_pos.x();
		l_distance[1] = l_size.y() - l_pos.y();
		l_distance[4] = l_size.y() + l_pos.y();
		l_distance[2] = l_size.z() - l_pos.z();
		l_distance[3] = l_size.z() + l_pos.z();
		
		// find the closest wall
		double l_minDistance = l_distance[0];
		int l_direction = 0;
		
		for (int i=1; i<6; i++)
		{
			if(l_distance[i] < l_minDistance)
			{
				l_minDistance = l_distance[i];
				l_direction = i;
			}
		}
		
		etVector l_force;
		if ( l_minDistance <= 0 ) // Not touching the Box
		{
			// l_force left as (0,0,0)
			m_previousDirection = -1;
			doFriction = false;
		}
		else
		{
			// if we were just touching the opposite side,
			// keep touching it
			
			// Check for l_direction opposite to m_previousDirection
			if( (l_direction + m_previousDirection) == 5 )
			{
				l_direction = m_previousDirection;
				l_minDistance = l_distance[l_direction];
			}
			
			m_previousDirection = l_direction;
			
			// forces
			switch (l_direction)
			{
			case 0:
				l_force = etVector(1,0,0);
				scp = etVector(l_size.x(),l_pos.y(),l_pos.z());
				break;
			case 5:
				l_force = etVector(-1,0,0);
				scp = etVector(-l_size.x(),l_pos.y(),l_pos.z());
				break;
			case 1:
				l_force = etVector(0,1,0);
				scp = etVector(l_pos.x(),l_size.y(),l_pos.z());
				break;
			case 4:
				l_force = etVector(0,-1,0);
				scp = etVector(l_pos.x(),-l_size.y(),l_pos.z());
				break;
			case 2:
				l_force = etVector(0,0,1);
				scp = etVector(l_pos.x(),l_pos.y(),l_size.z());
				break;
			case 3:
				l_force = etVector(0,0,-1);
				scp = etVector(l_pos.x(),l_pos.y(),-l_size.z());
				break;
			}
			
			l_force = l_force.scale(l_minDistance);
			//Calculate Friction	
			if (!doFriction) {
				// Set start values.
				setLastStylusPosition(l_pos);
				setLastStickPoint(scp);
				doFriction = true;
			}
			else {
				// Calculate friction.
				etVector frictionForce(0,0,0);
				etVector distance = scp - getLastStickPoint();
				double scaleFactor = l_force.length() * 10;
				float threshold_one = 0.01;
				
				if (distance.length() < threshold_one) {
					// No friction for small movements.
				}
				else if (distance.length() < staticFriction) {
					// Static friction.
					distance = distance - distance.normalize().scale(threshold_one);
					frictionForce = distance.scale(-1.0);
					if (scaleFactor > 1)
						scaleFactor = 1;
					frictionForce = frictionForce.scale(scaleFactor);
					l_force += frictionForce;
				}
				else {
					// Dynamic friction.
					stickPoint = getLastStickPoint()+distance.scale(0.05);
					setLastStickPoint(stickPoint);
					frictionForce = distance.normalize().scale(-dynamicFriction);
					if (scaleFactor > 1)
						scaleFactor = 1;
					frictionForce = frictionForce.scale(scaleFactor);
					l_force += frictionForce;
				}
			}
			setLastSCP(scp);
			l_force = l_force.scale(this->getStiffnessRaw());
			
		}

		return l_force;
	}
	else {
		doFriction = false;
		m_previousDirection = -1;
		return etVector(0,0,0);
	}
}

// See etForceRenderer.
bool etBox::cull(etVector l_pos)
{
	
    etVector l_size = this->getSizeRaw().scale(0.5);
	
    bool l_cull = (fabs(l_pos.x()) > fabs(l_size.x()) ||
		fabs(l_pos.y()) > fabs(l_size.y()) ||
		fabs(l_pos.z()) > fabs(l_size.z()));
	
    if (l_cull)
    {
        m_previousDirection = -1;
    }
	
    return l_cull;
}

// Constructor.
etBox::etBox()
: etObject(),
m_Size(DEFAULT_SIZE),
m_previousDirection(-1)
{
}

// Copy constructor.
etBox::etBox(const etBox& other)
: etObject(other),
m_Size(other.getSize())
{
}

// Assignment operator.
etBox& etBox::operator=(const etBox& other)
{
    etObject::operator=(other);
    setSize(other.getSize());
    return (*this);
}

// Returns the tool associated with the graphic renderer.
// See etObject
etBox* etBox::clone() const
{
    return new etBox(*this);
}
