#include "object.h"

const etVector DEFAULT_POSITION(0,0,0);
const etOrientation DEFAULT_ORIENTATION(0,1,0,0);
const double DEFAULT_STIFFNESS = 500.0;
const double DEFAULT_STATIC_FRICTION = 0.1;
const double DEFAULT_DYNAMIC_FRICTION = 0.08;

// Constructor.
etObject::etObject()
    : m_Position(DEFAULT_POSITION),
      m_Orientation(DEFAULT_ORIENTATION),
      m_Stiffness(DEFAULT_STIFFNESS),
	  staticFriction(DEFAULT_STATIC_FRICTION),
	  dynamicFriction(DEFAULT_DYNAMIC_FRICTION),
	  doFriction(false)
{
    // Do nothing.
}

// Constructor.
etObject::etObject(const etObject& other)
    : m_Position(other.getPosition()),
      m_Orientation(other.getOrientation()),
      m_Stiffness(other.getStiffness()),
	  staticFriction(DEFAULT_STATIC_FRICTION),
	  dynamicFriction(DEFAULT_DYNAMIC_FRICTION), 
	  doFriction(false)
{
    // Do nothing. 
}

// Copy constructor.
etObject& etObject::operator=(const etObject& other)
{
    setPosition(other.getPosition());
    setOrientation(other.getOrientation());
    setStiffness(other.getStiffness());
	staticFriction = DEFAULT_STATIC_FRICTION;
	dynamicFriction = DEFAULT_DYNAMIC_FRICTION;
	doFriction = false;
    return *this;
}

// Destructor.
etObject::~etObject()
{

}

// Set friction.
void etObject::setFriction(double newFriction) {
	if(newFriction<0)
		newFriction = 0;
	else if(newFriction>1)
		newFriction = 1;
	staticFriction = newFriction * 0.1;
	dynamicFriction = staticFriction * 0.8;
}

// Set last position.
void etObject::setLastStylusPosition(etVector pos) {
	lastPosition = pos;
}

// Get last position.
etVector etObject::getLastStylusPosition() {
	return lastPosition;
}

// Get last friction force.
etVector etObject::getLastFrictionForce() {
	return frictionForce;
}

// Set last position.
void etObject::setLastFrictionForce(etVector force) {
	frictionForce = force;
}

// Get last stick point.
etVector etObject::getLastStickPoint() {
	return stick_point;
}

// Get last stick point.
void etObject::setLastStickPoint(etVector sp) {
	stick_point = sp;
}

// Get last surface contact point.
etVector etObject::getLastSCP() {
	return scp;
}

// Set last surface contact point.
void etObject::setLastSCP(etVector scp) {
	this->scp = scp;
}