#ifndef ET_OBJECT_H
#define ET_OBJECT_H

// Includes
#include <et/variable.h>
#include <et/vector.h>
#include "orientation.h"//<et/orientation.h>



//
// Class:
//     etObject
//
// Description:
//     Base class for rendered objects in the e-Touch framework.
//
// Exceptions:
//     None
//
// Notes:
//     Figure out how to do modifier support
//
// See Also Classes:
//     None
//
// See Also Functions:
//     None
//

class etObject
{
	
	// ----- Object state accessor methods -----
	
	// Position accessor (get/setPosition())
	ET_SHARED_VARIABLE(etVector,Position);
	
	// Orientation accessor (get/setOrientation())
	ET_SHARED_VARIABLE(etOrientation,Orientation);
	
	// Stiffness accessor (get/setStiffness())
	ET_SHARED_VARIABLE(double,Stiffness);

	
public:

	// Set the Friction for the object.
	void setFriction(double newFriction);

	// Compute the force
	virtual etVector computePointForce(etVector l_position) = 0;

	void setRawStiffness(double newStiffness);

protected:
	
	// Constructor.
	etObject();
	
	// Copy constructor. 
	etObject(const etObject& other);
	
	// Assignment operator.
	etObject& operator=(const etObject& other);
	
	// Destructor.
	virtual ~etObject();

	// Set Last position.
	void setLastStylusPosition(etVector pos);

	// Get Last position.
	etVector getLastStylusPosition();

	void setLastFrictionForce(etVector force);

	etVector getLastFrictionForce();

	etVector getLastStickPoint();

	void setLastStickPoint(etVector sp);


	etVector getLastSCP();

	void setLastSCP(etVector scp);
	
	double staticFriction;
	double dynamicFriction;
	bool doFriction;
	
private:


	// Friction force.
	etVector frictionForce;

	etVector stick_point;

	etVector scp;

	// Last Stylus Position
	etVector lastPosition;

	// Last Time
	double lastTime;
};

#endif
