#include <stdio.h>
#include "sphere.h"

// Default size, resolution and color of an etSphere.
const double DEFAULT_RADIUS = 0.025; // 50mm diameter.

// Returns the force to be applied. 
etVector etSphere::computePointForce(etVector l_position)
{
    //etVector l_position = toLocal(a_point).getPosition();
	
    double l_distance = l_position.length();
    double l_radius = this->getRadiusRaw();
	
    etVector l_force;
    if (l_distance < l_radius)
    {
		scp = l_position.normalize().scale(l_radius);
        double l_magnitude = l_radius - l_distance;
        l_force = l_position;
		
        l_force = l_force.normalize();
        l_force = l_force.scale(l_magnitude);

		//Calculate Friction	
		if (!doFriction) {
			// Set start values.
			setLastStylusPosition(l_position);
			setLastStickPoint(scp);
			doFriction = true;
		}
		else {
			// Calculate friction.
			etVector frictionForce(0,0,0);
			etVector distance = scp - getLastStickPoint();
			double scaleFactor = l_force.length() * 10;
			float threshold_one = 0.01;
			
			if (distance.length() < threshold_one) {
				// No friction for small movements.
			}
			else if (distance.length() < staticFriction) {
				// Static friction.
				distance = distance - distance.normalize().scale(threshold_one);
				frictionForce = distance.scale(-1.0);
				if (scaleFactor > 1)
					scaleFactor = 1;
				frictionForce = frictionForce.scale(scaleFactor);
				l_force += frictionForce;
			}
			else {
				// Dynamic friction.
				stickPoint = getLastStickPoint()+distance.scale(0.05);
				setLastStickPoint(stickPoint);
				frictionForce = distance.normalize().scale(-dynamicFriction);
				if (scaleFactor > 1)
					scaleFactor = 1;
				frictionForce = frictionForce.scale(scaleFactor);
				l_force += frictionForce;
			}
		}
		setLastSCP(scp);
        // OPTIMIZATION:
        // l_force = l_force.scale(l_magnitude / l_distance);
        l_force = l_force.scale(this->getStiffnessRaw());
    }
	
    return l_force;
}

// Cull.
bool etSphere::cull(etVector l_position)
{
    double l_distance = l_position.length();
    double l_radius = this->getRadiusRaw();
	
    return l_distance > l_radius;
}

// Constructor.
etSphere::etSphere()
: etObject(),
m_Radius(DEFAULT_RADIUS)
{
}

// Copy constructor.
etSphere::etSphere(const etSphere& other)
: etObject(other),
m_Radius(other.getRadius())
{
}

// Assignment operator.
etSphere& etSphere::operator=(const etSphere& other)
{
    etObject::operator=(other);
    setRadius(other.getRadius());
    return (*this);
}

// Returns the tool associated with the graphic renderer.
// See etObject.
etSphere* etSphere::clone() const
{
    return new etSphere(*this);
}
