#include "trisurface.h"
#include "liquidtrisurface.h"
#include "box.h"
#include "sphere.h"
#include "cylinder.h"
#include "octree.h"
#include <et/lock.h>

static const double DEFAULT_STIFFNESS = 500;
static const double DEFAULT_FRICTION = 0.5;

class World{
public:
	bool collision;

	World();
	~World();
	void setMeshList(const iMeshList* csMeshList);
	etVector calculateForces(etVector point);
	std::pair<int,csVector3> getDynamicForcePair() const;
	std::list<std::pair<int,csVector3> > getDynamicForceList() const;
	void setForcesOn(bool on);


private:
	int nbrOfDynamicObjects;
	int nbrOfStaticObjects;
	bool forcesOn;
	WorldOctree* octree;
	const iMeshList* meshList;
	std::vector<etObject*> staticObjects;
	std::vector<etObject*> dynamicObjects;
	std::vector<int> dynamicIndices;
	std::vector<csBox3> boxes;
	std::vector<etObject*> wallsVector;
	std::list<etObject*> wallsList;
	std::pair<int,csVector3> dynamicForcePair;
	std::list<std::pair<int,csVector3> > dynamicForceList;
	etLock* mutex;

	void addStaticObjects(std::vector<etObject*> objects,std::vector<csBox3> boundBoxes,std::vector<double> minMax);
	void addDynamicObjects(std::vector<etObject*> objects, std::vector<int> dynamicIndexVector);
	std::list<int> getNearbyStaticObjects(double x, double y, double z) const;
	bool checkLimit(etVector force);
};
